//------------------------------------------------------------------------------
// Lamp : Open source game middleware
// Copyright (C) 2004  Junpei Ohtani ( Email : junpee@users.sourceforge.jp )
//
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public
// License as published by the Free Software Foundation; either
// version 2.1 of the License, or (at your option) any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public
// License along with this library; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//------------------------------------------------------------------------------

/** @file
 * マウスヘッダ
 * @author Junpee
 */

#ifndef MOUSE_H_
#define MOUSE_H_

#include <Input/Mouse/MouseState.h>
#include <Input/Mouse/MouseDevice.h>

namespace Lamp{

//------------------------------------------------------------------------------
/**
 * マウス
 */
class Mouse{
friend class LampInput;
friend class BufferedInput;
public:
	//--------------------------------------------------------------------------
	// 定数
	//--------------------------------------------------------------------------
	/// 最大ボタン数
	static const int maxButtonCount = 8;

	//--------------------------------------------------------------------------
	// 軸
	//--------------------------------------------------------------------------
	/**
	 * X軸の取得
	 * @return X軸
	 */
	virtual int getXAxis() const{ return state_.getXAxis(); }

	/**
	 * Y軸の取得
	 * @return Y軸
	 */
	virtual int getYAxis() const{ return state_.getYAxis(); }

	/**
	 * Z軸の取得
	 * @return Z軸
	 */
	virtual int getZAxis() const{ return state_.getZAxis(); }

	/**
	 * Z解像度の取得
	 * @return Z解像度
	 */
	int getZResolution() const{ return device_->getZResolution(); }

	/**
	 * 補正済みZ軸の取得
	 * @return 補正済みZ軸
	 */
	virtual float getCorrectedZAxis() const{
		return (float)getZAxis() / (float)getZResolution();
	}

	//--------------------------------------------------------------------------
	// ボタン
	//--------------------------------------------------------------------------
	/**
	 * ボタン数の取得
	 * @return ボタン数
	 */
	virtual int getButtonCount() const{ return device_->getButtonCount(); }

	/**
	 * ボタンが押されているか
	 * @param id 対象ボタンID
	 * @return ボタンが押されていればtrue
	 */
	virtual bool buttonPressed(int id) const{
		return state_.buttonPressed(id);
	}

	/**
	 * ボタンが下がった
	 * @param id ボタンID
	 * @return ボタンが下がったならばtrue
	 */
	virtual bool buttonDown(int id) const{
		return (state_.buttonPressed(id) && (!preState_.buttonPressed(id)));
	}

	/**
	 * ボタンが上がった
	 * @param id ボタンID
	 * @return ボタンが上がったならばtrue
	 */
	virtual bool buttonUp(int id) const{
		return ((!state_.buttonPressed(id)) && preState_.buttonPressed(id));
	}

	//--------------------------------------------------------------------------
	// クリック
	//--------------------------------------------------------------------------
	/**
	 * クリック位置範囲の設定
	 * @param clickPositionRange クリック位置範囲
	 */
	virtual void setClickPositionRange(int clickPositionRange){
		clickPositionRange_ = clickPositionRange;
	}

	/**
	 * クリック位置範囲の取得
	 * @return クリック位置範囲
	 */
	virtual int getClickPositionRange() const{ return clickPositionRange_; }

	//--------------------------------------------------------------------------
	/**
	 * ダブルクリック時間範囲の設定
	 * @param doubleClickTimeRange ダブルクリック時間範囲
	 */
	virtual void setDoubleClickTimeRange(int doubleClickTimeRange){
		doubleClickTimeRange_ = doubleClickTimeRange;
	}

	/**
	 * ダブルクリック時間範囲の取得
	 * @return ダブルクリック時間範囲
	 */
	virtual int getDoubleClickTimeRange() const{ return doubleClickTimeRange_; }

	//--------------------------------------------------------------------------
	/**
	 * クリックされたか
	 * @param id 対象ボタンID
	 * @return ボタンがクリックされていればtrue
	 */
	virtual bool clicked(int id) const{
		Assert(id >= 0);
		Assert(id < maxButtonCount);
		return clicked_[id];
	}

	/**
	 * ダブルクリックされたか
	 * @param id 対象ボタンID
	 * @return ボタンがダブルクリックされていればtrue
	 */
	virtual bool doubleClicked(int id) const{
		Assert(id >= 0);
		Assert(id < maxButtonCount);
		return doubleClicked_[id];
	}

	/**
	 * ダブルダウンされたか
	 * @param id 対象ボタンID
	 * @return ボタンがダブルダウンされていればtrue
	 */
	virtual bool doubleDown(int id) const{
		Assert(id >= 0);
		Assert(id < maxButtonCount);
		return doubleDown_[id];
	}

	//--------------------------------------------------------------------------
	/**
	 * 名前の取得
	 * @return 名前
	 */
	virtual String getName() const{ return device_->getProductName(); }

	/**
	 * アタッチされているか
	 * @return アタッチされていればtrue
	 */
	virtual bool isAttached() const{ return device_->isAttached(); }

	/**
	 * ポーリングが必要か
	 * @return ポーリングが必要ならtrue
	 */
	virtual bool isPolled() const{ return device_->isPolled(); }

	/**
	 * 文字列への変換
	 * @return 文字列
	 */
	virtual String toString() const;

	/**
	 * クリア
	 */
	virtual void clear(){
		state_.clear();
		preState_.clear();
	}

	/**
	 * 協調レベルの設定
	 * @param exclusive 排他モードならtrue
	 * @param foreground フォアグラウンドモードならtrue
	 * @return 成功すればtrue
	 */
	virtual bool setCooperativeLevel(bool exclusive, bool foreground){
		return device_->setCooperativeLevel(exclusive, foreground);
	}

	/**
	 * 排他モードか
	 * @return 排他モードならtrue
	 */
	virtual bool isExclusive() const{ return device_->isExclusive(); }

	/**
	 * フォアグラウンドモードか
	 * @return フォアグラウンドモードならtrue、バックグラウンドモードならfalse
	 */
	virtual bool isForeground() const{ return device_->isForeground(); }

protected:
	//--------------------------------------------------------------------------
	/**
	 * コンストラクタ
	 * @param device デバイス
	 */
	Mouse(MouseDevice* device);

	/**
	 * デストラクタ
	 */
	virtual ~Mouse();

	/**
	 * 次のステート設定
	 * @param state 次のステート
	 */
	virtual void setNextState(const MouseState& state);

	/**
	 * ステートの取得
	 * @return ステート
	 */
	virtual const MouseState& getState(){ return state_; }

	/**
	 * クリックステートクリア
	 */
	virtual void clickStateClear();

private:
	//--------------------------------------------------------------------------
	// デバイス
	MouseDevice* device_;
	// ステート
	MouseState state_;
	// 前回のステート
	MouseState preState_;
	// 押し下げオフセット
	Point2i downOffset_[maxButtonCount];
	// 押し下げ時間
	int downTime_[maxButtonCount];
	// 前回の押し下げオフセット
	Point2i preDownOffset_[maxButtonCount];
	// 前回の押し下げ時間
	int preDownTime_[maxButtonCount];
	// クリック位置範囲
	int clickPositionRange_;
	// ダブルクリック時間範囲
	int doubleClickTimeRange_;
	// クリック
	bool clicked_[maxButtonCount];
	// ダブルクリック
	bool doubleClicked_[maxButtonCount];
	// ダブルダウン
	bool doubleDown_[maxButtonCount];
};

//------------------------------------------------------------------------------
} // End of namespace Lamp
#endif // End of MOUSE_H_
//------------------------------------------------------------------------------
